@@ -33,7 +33,8 @@ function _createExtensionType(name, folder, manager, overrides){
         roles: {},
         callbacks: {
             finishExtensionLoad: manager.finishExtensionLoad,
-            prepareExtensionUnload: manager.prepareExtensionUnload
+            prepareExtensionUnload: manager.prepareExtensionUnload,
+            prepareExtensionReload: manager.prepareExtensionReload
         },
         userDir: GLib.build_filenamev([global.userdatadir, folder]),
         legacyMeta: {}
@@ -86,7 +87,8 @@ var Type = {
         roles: {
             notifications: null,
             windowlist: null,
-            panellauncher: null
+            panellauncher: null,
+            tray: null
         }
     }),
     DESKLET: _createExtensionType("Desklet", "desklets", DeskletManager, {
@@ -158,17 +160,22 @@ function Extension(type, uuid) {
     if (extension) {
         return Promise.resolve(true);
     }
+    let force = false;
+    if (uuid.substr(0, 1) === '!') {
+        uuid = uuid.replace(/^!/, '');
+        force = true;
+    }
     let dir = findExtensionDirectory(uuid, type.userDir, type.folder);
 
     if (dir == null) {
         forgetExtension(uuid, type, true);
         return Promise.resolve(null);
     }
-    return this._init(dir, type, uuid);
+    return this._init(dir, type, uuid, force);
 }
 
 Extension.prototype = {
-    _init: function(dir, type, uuid) {
+    _init: function(dir, type, uuid, force) {
         this.name = type.name;
         this.uuid = uuid;
         this.dir = dir;
@@ -215,8 +222,8 @@ Extension.prototype = {
             // https://github.com/linuxmint/cjs/blob/055da399c794b0b4d76ecd7b5fabf7f960f77518/modules/_lie.js#L9
             startTime = new Date().getTime();
             this.meta = meta;
-            if (uuid.indexOf('!') !== 0) {
-                this.uuid = uuid.replace(/^!/, '');
+
+            if (!force) {
                 this.validateMetaData();
             }
 
@@ -482,7 +489,7 @@ function loadExtension(uuid, type) {
  * @type (Extension.Type): type of xlet
  * @deleteConfig (bool): delete also config files, defaults to true
  */
-function unloadExtension(uuid, type, deleteConfig = true) {
+function unloadExtension(uuid, type, deleteConfig = true, reload = false) {
     let extensionIndex = queryCollection(extensions, {uuid}, true);
     if (extensionIndex > -1) {
         let extension = extensions[extensionIndex];
@@ -492,6 +499,9 @@ function unloadExtension(uuid, type, deleteConfig = true) {
         // but it will be removed on next reboot, and hopefully nothing
         // broke too much.
         try {
+            if (reload) {
+                Type[extension.upperType].callbacks.prepareExtensionReload(extension);
+            }
             Type[extension.upperType].callbacks.prepareExtensionUnload(extension, deleteConfig);
         } catch (e) {
             logError(`Error disabling ${extension.lowerType} ${extension.uuid}`, extension.uuid, e);
@@ -528,7 +538,7 @@ function forgetExtension(extensionIndex, uuid, type, forgetMeta) {
  */
 function reloadExtension(uuid, type) {
     if (getExtension(uuid)) {
-        unloadExtension(uuid, type, false);
+        unloadExtension(uuid, type, false, true);
         Main._addXletDirectoriesToSearchPath();
         loadExtension(uuid, type);
         return;
